@@ -30,7 +30,6 @@ from pywayland.server import Client, Listener
 from wlroots import PtrHasData
 from wlroots.util.box import Box
 from wlroots.wlr_types import Buffer
-from wlroots.wlr_types.idle_inhibit_v1 import IdleInhibitorV1
 from wlroots.wlr_types.pointer_constraints_v1 import (
     PointerConstraintV1,
     PointerConstraintV1StateField,
@@ -54,8 +53,6 @@ except ModuleNotFoundError:
 if typing.TYPE_CHECKING:
     from typing import Any
 
-    from wlroots.wlr_types import Surface
-
     from libqtile.backend.wayland.core import Core
     from libqtile.command.base import CommandObject, ItemT
     from libqtile.core.manager import Qtile
@@ -115,7 +112,6 @@ class Window(typing.Generic[S], _Base, base.Window, HasListeners):
         self.y = 0
         self._opacity: float = 1.0
         self._wm_class: str | None = None
-        self._idle_inhibitors_count: int = 0
         self._urgent = False
 
         # Create a scene-graph tree for this window and its borders
@@ -233,15 +229,6 @@ class Window(typing.Generic[S], _Base, base.Window, HasListeners):
         logger.debug("Signal: foreign_toplevel_management request_close")
         self.kill()
 
-    def _on_inhibitor_destroy(self, listener: Listener, surface: Surface) -> None:
-        # We don't have reference to the inhibitor, but it doesn't really
-        # matter we only need to keep count of how many inhibitors there are
-        self._idle_inhibitors_count -= 1
-        listener.remove()
-        if self._idle_inhibitors_count == 0:
-            self.core.check_idle_inhibitor()
-            # TODO: do we also need to check idle inhibitors when unmapping?
-
     def hide(self) -> None:
         self.container.node.set_enabled(enabled=False)
         seat = self.core.seat
@@ -301,7 +288,17 @@ class Window(typing.Generic[S], _Base, base.Window, HasListeners):
             if self.group.screen:
                 # for floats remove window offset
                 self.x -= self.group.screen.x
+            group_ref = self.group
             self.group.remove(self)
+            # delete groups with `persist=False`
+            if (
+                not self.qtile.dgroups.groups_map[group_ref.name].persist
+                and len(group_ref.windows) <= 1
+            ):
+                # set back original group so _del() can grab it
+                self.group = group_ref
+                self.qtile.dgroups._del(self)
+                self.group = None
 
         if group.screen and self.x < group.screen.x:
             self.x += group.screen.x
@@ -389,15 +386,8 @@ class Window(typing.Generic[S], _Base, base.Window, HasListeners):
 
     @opacity.setter
     def opacity(self, opacity: float) -> None:
-        if opacity < 1.0:
-            logger.warning(
-                "Sorry, transparency is not yet supported by the wlroots API used by "
-                "Qtile. Transparency can only be achieved if the client sets it."
-            )
-            # wlroots' scene graph doesn't support setting the opacity of trees/nodes by
-            # the compositor. See: https://gitlab.freedesktop.org/wlroots/wlroots/-/issues/3393
-            # If/when that becomes supported, this warning can be removed.
         self._opacity = opacity
+        self.core.configure_node_opacity(self.container.node)
 
     @property
     def floating(self) -> bool:
@@ -594,27 +584,9 @@ class Window(typing.Generic[S], _Base, base.Window, HasListeners):
             fullscreen=self._float_state == FloatStates.FULLSCREEN,
         )
 
-    def match(self, match: config.Match) -> bool:
+    def match(self, match: config._Match) -> bool:
         return match.compare(self)
 
-    def add_idle_inhibitor(
-        self,
-        surface: Surface,
-        _x: int,
-        _y: int,
-        inhibitor: IdleInhibitorV1,
-    ) -> None:
-        if surface == inhibitor.surface:
-            self._idle_inhibitors_count += 1
-            inhibitor.data = self
-            self.add_listener(inhibitor.destroy_event, self._on_inhibitor_destroy)
-            if self._idle_inhibitors_count == 1:
-                self.core.check_idle_inhibitor()
-
-    @property
-    def is_idle_inhibited(self) -> bool:
-        return self._idle_inhibitors_count > 0
-
     def _items(self, name: str) -> ItemT:
         if name == "group":
             return True, []
@@ -791,7 +763,6 @@ class Static(typing.Generic[S], _Base, base.Static, HasListeners):
         qtile: Qtile,
         surface: S,
         wid: int,
-        idle_inhibitor_count: int = 0,
     ):
         base.Static.__init__(self)
         self.core = core
@@ -807,7 +778,6 @@ class Static(typing.Generic[S], _Base, base.Static, HasListeners):
         self.bordercolor: list[ffi.CData] = [_rgb((0, 0, 0, 1))]
         self.opacity: float = 1.0
         self._wm_class: str | None = None
-        self._idle_inhibitors_count = idle_inhibitor_count
         self.ftm_handle: ftm.ForeignToplevelHandleV1 | None = None
         self.data_handle = ffi.new_handle(self)
         surface.data = self.data_handle
@@ -872,32 +842,6 @@ class Static(typing.Generic[S], _Base, base.Static, HasListeners):
         logger.debug("Signal: foreign_toplevel_management static request_close")
         self.kill()
 
-    def _on_inhibitor_destroy(self, listener: Listener, surface: Surface) -> None:
-        # We don't have reference to the inhibitor, but it doesn't really
-        # matter we only need to keep count of how many inhibitors there are
-        self._idle_inhibitors_count -= 1
-        listener.remove()
-        if self._idle_inhibitors_count == 0:
-            self.core.check_idle_inhibitor()
-
-    def add_idle_inhibitor(
-        self,
-        surface: Surface,
-        _x: int,
-        _y: int,
-        inhibitor: IdleInhibitorV1,
-    ) -> None:
-        if surface == inhibitor.surface:
-            self._idle_inhibitors_count += 1
-            inhibitor.data = self
-            self.add_listener(inhibitor.destroy_event, self._on_inhibitor_destroy)
-            if self._idle_inhibitors_count == 1:
-                self.core.check_idle_inhibitor()
-
-    @property
-    def is_idle_inhibited(self) -> bool:
-        return self._idle_inhibitors_count > 0
-
     def belongs_to_client(self, other: Client) -> bool:
         return other == Client.from_resource(self.surface.surface._ptr.resource)
 
