@@ -1489,6 +1489,13 @@ class _Window:
         self.window.set_property("_NET_WM_STATE", reply)
         self.change_layer(up=False)
 
+    @expose_command()
+    def bring_to_front(self):
+        if self.get_wm_type() != "desktop":
+            self.window.configure(stackmode=xcffib.xproto.StackMode.Above)
+            self.raise_children()
+            self.qtile.core.update_client_lists()
+
 
 class Internal(_Window, base.Internal):
     """An internal window, that should not be managed by qtile"""
@@ -1660,13 +1667,6 @@ class Static(_Window, base.Static):
         if name == "_NET_WM_STRUT_PARTIAL":
             self.update_strut()
 
-    @expose_command()
-    def bring_to_front(self):
-        if self.get_wm_type() != "desktop":
-            self.window.configure(stackmode=xcffib.xproto.StackMode.Above)
-            self.raise_children()
-            self.qtile.core.update_client_lists()
-
 
 class Window(_Window, base.Window):
     _window_mask = (
@@ -2149,8 +2149,15 @@ class Window(_Window, base.Window):
                 focus_behavior = self.qtile.config.focus_on_window_activation
                 if focus_behavior == "focus":
                     logger.debug("Focusing window")
-                    self.qtile.current_screen.set_group(self.group)
-                    self.group.focus(self)
+                    # Windows belonging to a scratchpad need to be toggled properly
+                    if isinstance(self.group, ScratchPad):
+                        for dropdown in self.group.dropdowns.values():
+                            if dropdown.window is self:
+                                dropdown.show()
+                                break
+                    else:
+                        self.qtile.current_screen.set_group(self.group)
+                        self.group.focus(self)
                 elif focus_behavior == "smart":
                     if not self.group.screen:
                         logger.debug(
@@ -2159,8 +2166,15 @@ class Window(_Window, base.Window):
                         return
                     if self.group.screen == self.qtile.current_screen:
                         logger.debug("Focusing window")
-                        self.qtile.current_screen.set_group(self.group)
-                        self.group.focus(self)
+                        # Windows belonging to a scratchpad need to be toggled properly
+                        if isinstance(self.group, ScratchPad):
+                            for dropdown in self.group.dropdowns.values():
+                                if dropdown.window is self:
+                                    dropdown.show()
+                                    break
+                        else:
+                            self.qtile.current_screen.set_group(self.group)
+                            self.group.focus(self)
                     else:  # self.group.screen != self.qtile.current_screen:
                         logger.debug("Setting urgent flag for window")
                         self.urgent = True
@@ -2295,13 +2309,6 @@ class Window(_Window, base.Window):
     def disable_fullscreen(self):
         self.fullscreen = False
 
-    @expose_command()
-    def bring_to_front(self):
-        if self.get_wm_type() != "desktop":
-            self.window.configure(stackmode=xcffib.xproto.StackMode.Above)
-            self.raise_children()
-            self.qtile.core.update_client_lists()
-
     def _is_in_window(self, x, y, window):
         return window.edges[0] <= x <= window.edges[2] and window.edges[1] <= y <= window.edges[3]
 
