@@ -37,6 +37,7 @@
 
 from __future__ import annotations
 
+import contextlib
 import functools
 import operator
 from itertools import chain, repeat
@@ -54,6 +55,7 @@ from xcffib.xproto import CW, EventMask, WindowClass
 from libqtile.backend.x11 import window
 from libqtile.backend.x11.xcursors import Cursors
 from libqtile.backend.x11.xkeysyms import keysyms
+from libqtile.config import ScreenRect
 from libqtile.log_utils import logger
 from libqtile.utils import QtileError, hex
 
@@ -213,6 +215,13 @@ XCB_CONN_ERRORS = {
     7: "XCB_CONN_CLOSED_FDPASSING_FAILED",
 }
 
+# Some opcodes from xproto.h, used for faking input.
+XCB_KEY_PRESS = 2
+XCB_KEY_RELEASE = 3
+XCB_BUTTON_PRESS = 4
+XCB_BUTTON_RELEASE = 5
+XCB_MOTION_NOTIFY = 6
+
 
 class MaskMap:
     """
@@ -353,20 +362,6 @@ class Screen(_Wrapper):
                     return i.visuals[0]
 
 
-class PseudoScreen:
-    """
-    This may be a Xinerama screen or a RandR CRTC, both of which are
-    rectangular sections of an actual Screen.
-    """
-
-    def __init__(self, conn, x, y, width, height):
-        self.conn = conn
-        self.x = x
-        self.y = y
-        self.width = width
-        self.height = height
-
-
 class Colormap:
     def __init__(self, conn, cid):
         self.conn = conn
@@ -383,7 +378,12 @@ class Colormap:
             def x8to16(i):
                 return 0xFFFF * (i & 0xFF) // 0xFF
 
-            color = hex(color)
+            try:
+                color = hex(color)
+            except ValueError:
+                logger.error("Colormap failed to allocate %s", color)
+                color = "#ff0000"
+
             r = x8to16(int(color[-6] + color[-5], 16))
             g = x8to16(int(color[-4] + color[-3], 16))
             b = x8to16(int(color[-2] + color[-1], 16))
@@ -405,17 +405,13 @@ class RandR:
         self.ext.SelectInput(conn.default_screen.root.wid, xcffib.randr.NotifyMask.ScreenChange)
 
     def query_crtcs(self, root):
-        crtc_list = []
-        for crtc in self.ext.GetScreenResources(root).reply().crtcs:
-            crtc_info = self.ext.GetCrtcInfo(crtc, xcffib.CurrentTime).reply()
-            crtc_dict = {
-                "x": crtc_info.x,
-                "y": crtc_info.y,
-                "width": crtc_info.width,
-                "height": crtc_info.height,
-            }
-            crtc_list.append(crtc_dict)
-        return crtc_list
+        infos = []
+        for output in self.ext.GetScreenResources(root).reply().outputs:
+            output_info = self.ext.GetOutputInfo(output, xcffib.CurrentTime).reply()
+            crtc_info = self.ext.GetCrtcInfo(output_info.crtc, xcffib.CurrentTime).reply()
+
+            infos.append(ScreenRect(crtc_info.x, crtc_info.y, crtc_info.width, crtc_info.height))
+        return infos
 
 
 class XFixes:
@@ -435,42 +431,6 @@ class XFixes:
         self.conn.xfixes.ext.SelectSelectionInput(window.wid, _selection, self.selection_mask)
 
 
-class NetWmState:
-    """NetWmState is a descriptor for _NET_WM_STATE_* properties"""
-
-    def __init__(self, prop_name):
-        self.prop_name = prop_name
-
-    def __get__(self, xcbq_win, cls):
-        try:
-            atom = self.atom
-        except AttributeError:
-            atom = xcbq_win.conn.atoms[self.prop_name]
-            self.atom = atom
-        reply = xcbq_win.get_property("_NET_WM_STATE", "ATOM", unpack=int)
-        if atom in reply:
-            return True
-        return False
-
-    def __set__(self, xcbq_win, value):
-        try:
-            atom = self.atom
-        except AttributeError:
-            atom = xcbq_win.conn.atoms[self.prop_name]
-            self.atom = atom
-
-        value = bool(value)
-        reply = list(xcbq_win.get_property("_NET_WM_STATE", "ATOM", unpack=int))
-        is_set = atom in reply
-        if is_set and not value:
-            reply.remove(atom)
-            xcbq_win.set_property("_NET_WM_STATE", reply)
-        elif value and not is_set:
-            reply.append(atom)
-            xcbq_win.set_property("_NET_WM_STATE", reply)
-        return
-
-
 class Connection:
     _extmap = {
         "xinerama": Xinerama,
@@ -522,28 +482,19 @@ class Connection:
 
     @property
     def pseudoscreens(self):
-        pseudoscreens = []
         if hasattr(self, "xinerama"):
+            pseudoscreens = []
             for i, s in enumerate(self.xinerama.query_screens()):
-                scr = PseudoScreen(
-                    self,
+                scr = ScreenRect(
                     s.x_org,
                     s.y_org,
                     s.width,
                     s.height,
                 )
                 pseudoscreens.append(scr)
+            return pseudoscreens
         elif hasattr(self, "randr"):
-            for i in self.randr.query_crtcs(self.screens[0].root.wid):
-                scr = PseudoScreen(
-                    self,
-                    i["x"],
-                    i["y"],
-                    i["width"],
-                    i["height"],
-                )
-                pseudoscreens.append(scr)
-        return pseudoscreens
+            return self.randr.query_crtcs(self.screens[0].root.wid)
 
     def finalize(self):
         self.cursors.finalize()
@@ -627,10 +578,8 @@ class Connection:
         return window.XWindow(self, wid)
 
     def disconnect(self):
-        try:
+        with contextlib.suppress(xcffib.ConnectionException):
             self.conn.disconnect()
-        except xcffib.ConnectionException:
-            logger.error("Failed to disconnect, connection already failed?")
         self._connected = False
 
     def flush(self):
@@ -698,9 +647,13 @@ class Painter:
         width = max((win.x + win.width for win in root_windows))
         height = max((win.y + win.height for win in root_windows))
 
-        root_pixmap = self.default_screen.root.get_property(
-            "_XROOTPMAP_ID", xcffib.xproto.Atom.PIXMAP, int
-        )
+        try:
+            root_pixmap = self.default_screen.root.get_property(
+                "_XROOTPMAP_ID", xcffib.xproto.Atom.PIXMAP, int
+            )
+        except xcffib.ConnectionException:
+            root_pixmap = None
+
         if not root_pixmap:
             root_pixmap = self.default_screen.root.get_property(
                 "ESETROOT_PMAP_ID", xcffib.xproto.Atom.PIXMAP, int
